/**
 * @description vue3 实现个优雅的拖拽
 * https://juejin.cn/post/7231152541404594237
 *
 */
import { ref, onMounted, computed, unref, onBeforeUnmount } from "vue";

import type { Ref } from "vue";

export const defaultWindow = /*#__PURE__*/ window;

interface useDragOption {
  draggingElement?: any;

  draggingHandle?: any;

  initialValue?: any;

  axis?: "x" | "y" | "both";

  limitDOM?: Ref<HTMLElement | null | undefined>;
}

// Composables

const useDrag = (target: Ref<HTMLElement | null>, options: useDragOption,executeClick) => {
  const {
    draggingElement = defaultWindow,

    draggingHandle = target,

    initialValue,

    axis = "both",

    limitDOM,
  } = options;

  // 图标初始位置
  let _x = document.body.clientWidth - 100;
  let _y = document.body.clientHeight - 100;
  const position = ref(initialValue ?? { x: _x, y: _y });
  // 点击标记，拖动标记
  const dragFlag = ref(false)
  /**
   * @description 标记：是否可以点击
   *  pointerdown pointermove pointerup
   *  点击不会有移动步骤：pointermove
   *  拖动 3
   *  可以点击的 2
   */
  const dragMoveFlag = ref(0)

  const size = computed(() => {
    return {
      width: target?.value?.offsetWidth || 0,

      height: target?.value?.offsetHeight || 0,
    };
  }); // 移动的变化

  const pressedDelta = ref();

  onMounted(() => {
    // 判断是否有target

    if (!unref(target)) {
      console.warn("drag目标元素不存在，请检查！");

      return;
    } // 注册事件 此处可以搞个整体的事件,来提高一些性能,累了毁灭吧

    target.value!.addEventListener("pointerdown", start);
    draggingElement!.addEventListener("pointermove", move);
    draggingElement!.addEventListener("pointerup", end);

    // target.value!.addEventListener("mousedown", start);
    // draggingElement!.addEventListener("mousemove", move);
    // draggingElement!.addEventListener("mouseup", end);
  });

  onBeforeUnmount(() => {
    // 判断是否有target

    if (unref(target)) {
      // 取消事件

      target.value!.removeEventListener("pointerdown", start);

      draggingElement!.removeEventListener("pointermove", move);

      draggingElement!.removeEventListener("pointerup", end);
    }
  }); // 移动开始

  const start = (e: PointerEvent) => {
    console.log('~~~~~~start~~~~');
    dragFlag.value = true;
    dragMoveFlag.value = 1;

    const rect = target.value!.getBoundingClientRect(); // 记录下点击的位置与左上角的偏差

    const pos = {
      x: e.clientX - rect.left,

      y: e.clientY - rect.top,
    };

    pressedDelta.value = pos;
  }; // 移动事件

  const move = (e: PointerEvent) => {
    // 如果没点击开始则不触发
    if (!pressedDelta.value) {
      return;
    }

    console.log('~~~~~~move~~~~')
    dragMoveFlag.value = 2;
    let { x, y } = position.value; // 鼠标移动到的位置减去偏差
    console.log('~~x~~~~~',x,y);

    if (axis === "x" || axis === "both") {
      x = e.clientX - pressedDelta.value.x;
    }

    if (axis === "y" || axis === "both") {
      y = e.clientY - pressedDelta.value.y;
    }

    position.value = limitArea({ x, y });
  }; // 移动结束

  const end = (e: PointerEvent) => {


    dragMoveFlag.value += 1

    console.log('~~end~~~dragMoveFlag：',dragMoveFlag.value,'dragFlag:',dragFlag.value)
    /*
    * 执行点击事件
    *   dragMoveFlag 执行  begin end
    *   dragFlag begin设置为  true, move 设置为 false
    *   dragFlag 加上这个的原因是，点击 空白区间会累加 dragMoveFlag
    * */
    if(dragMoveFlag.value === 2 && !!dragFlag.value) {
      executeClick && executeClick()
    }

    // 标记
    dragFlag.value = false

    if (!pressedDelta.value) {
      return;
    }

    pressedDelta.value = undefined;
  }; // 要求不超过边界

  const limitArea = (
    {
      x = position.value.x,
      y = position.value.y,
    }: { x: number; y: number } = position.value
  ) => {
    if (x < areaLimit.value.startX) {
      x = areaLimit.value.startX;
    }

    if (x > areaLimit.value.endX - size.value.width) {
      x = areaLimit.value.endX - size.value.width;
    }

    if (y < areaLimit.value.startY) {
      y = areaLimit.value.startY;
    }

    if (y > areaLimit.value.endY - size.value.height) {
      y = areaLimit.value.endY - size.value.height;
    } // 如果元素小于限制区域 将元素移动到左上角

    if (areaLimit.value.endY - areaLimit.value.startY < size.value.height) {
      y = areaLimit.value.startY;
    }

    if (areaLimit.value.endX - areaLimit.value.startX < size.value.width) {
      x = areaLimit.value.startX;
    }

    return {
      x,
      y,
    };
  }; // 计算div的返回限制范围

  const areaLimit = ref({
    startX: 0,

    startY: 0,

    endX: 500,

    endY: 500,
  });

  let mo: MutationObserver;

  let re: ResizeObserver; // 更新相对位置

  const initWatch = () => {
    const limitdiv = unref(limitDOM)
      ? unref(limitDOM)
      : document.getElementById("app");

    const callback = () => {
      const { left, right, top, bottom } = limitdiv!.getBoundingClientRect();

      areaLimit.value = {
        startX: left,

        startY: top,

        endX: right,

        endY: bottom,
      };

      position.value = limitArea();
    };

    if (limitdiv) {
      mo = new MutationObserver(callback);

      mo.observe(limitdiv, {
        attributes: true,
      });

      re = new ResizeObserver(callback);

      re.observe(limitdiv);

      callback();
    }
  };

  onMounted(() => {
    initWatch();
  });

  onBeforeUnmount(() => {
    if (mo) {
      mo?.disconnect();
    }

    if (re) {
      re?.disconnect();
    }
  });

  return {
    dragFlag,

    draggingHandle,

    x: computed(() => position.value.x),

    y: computed(() => position.value.y),

    position,

    isDragging: computed(() => !!pressedDelta.value),

    style: computed(
      () => `left:${position.value.x}px;top:${position.value.y}px;`
    ),
  };
};

export { useDrag };
