import { onBeforeUnmount, ref, Ref, computed } from 'vue'
import { __POWERED_BY_WUJIE__ } from '@core/utils/wujie/wujieSub'
import useDraggable from './useDraggable'

export default function useWujieLuckySheet({ showOnlineExcel }: { showOnlineExcel: Ref<boolean> }) {
	if (!__POWERED_BY_WUJIE__) {
		return {}
	}
	const luckySheetMenuToolRef = ref<HTMLElement | null>(null)
	const toolTipPosition = ref<{
		top: number
		left: number
		right: number
		bottom: number
		width: number
		x: number
		y: number
	} | null>(null)
	const isShow = computed(() => showOnlineExcel.value)

	const setLuckSheetStyle = () => {
		const head = document.querySelector('head')
		let styleEle = document.createElement('style')
		styleEle.textContent = `
            body.tg-full-excel-body {
                top: 0 !important;
                left: 0 !important;
                width: 100vw !important;
                height: 100vh !important;
            }
            #luckysheet-tooltip-up {
                display: none;
            }
            .luckysheet-wa-editor {
                min-height: 32px;
                height: auto;
                white-space: initial;
            }
            .luckysheet-modal-dialog {
                position: initial;
                padding: 0;
                background-color: initial;
                border: none;
                box-shadow: none;
                pointer-events: none;
            }
            .luckysheet-modal-dialog-new {
                position: absolute;
                left: 50%;
                top: 50%;
                transform: translate(-50%, -50%);
                padding: 30px 42px;
                background-color: #fff;
                border: 1px solid rgba(0,0,0,.333);
                box-shadow: 0 4px 16px rgba(0,0,0,.2);
                z-index: 9999;
            }
            .luckysheet-sheet-area 
            div.luckysheet-sheet-container 
            div.luckysheet-sheet-container-c {
                display: inline-flex;
            }

            #luckysheet-sheet-list {
                left: 70px !important;
            }
            #luckysheet-rightclick-sheet-menu {
                bottom: 60px !important;
            }
        `
		head?.appendChild(styleEle)
	}
	setLuckSheetStyle()

	const bodyClassName = 'tg-full-excel-body'
	const bodyClassList = document.body.classList
	const setBodyStyle = () => {
		bodyClassList.add(bodyClassName)
	}

	const luckySheetCreatObserver = ref<MutationObserver | null>(null)
	const menuToolVisibleObserver = ref<IntersectionObserver | null>(null)
	/**
	 * The function `handleLuckySheetCreat` handles mutations in a list and calls another function
	 * `handleAddedNode` for each added node.
	 * @param {MutationRecord[]} mutationsList - An array of MutationRecord objects that represent the
	 * changes to the DOM.
	 * @param {string} unbornSelector - The `unbornSelector` parameter is a string that represents a CSS
	 * selector. It is used to select elements that are not yet added to the DOM.
	 */
	const handleLuckySheetCreat = (mutationsList: MutationRecord[], unbornSelector: string) => {
		mutationsList.forEach(mutation => {
			if (mutation.type !== 'childList') {
				return
			}
			Array.from(mutation.addedNodes).forEach(node => handleAddedNode(node, unbornSelector))
		})
	}

	/**
	 * The function `handleAddedNode` assigns the value of
	 * `node.querySelector<HTMLDivElement>(unbornSelector)` to `luckySheetMenuToolRef.value` if `node` is
	 * an instance of `HTMLElement`, and then calls `onLuckySheetCreated` with
	 * `luckySheetMenuToolRef.value` as an argument.
	 * @param {Node} node - The `node` parameter is the node that has been added to the DOM. It can be any
	 * type of node, but in this case, it is expected to be an instance of `HTMLElement`.
	 * @param {string} unbornSelector - The `unbornSelector` parameter is a string that represents a CSS
	 * selector. It is used to select an element within the `node` parameter that is of type
	 * `HTMLElement`.
	 * @returns If the `node` is not an instance of `HTMLElement`, then `undefined` is being returned.
	 */
	const handleAddedNode = (node: Node, unbornSelector: string) => {
		if (!(node instanceof HTMLElement)) {
			return
		}
		luckySheetMenuToolRef.value = node.querySelector<HTMLDivElement>(unbornSelector)
		if (luckySheetMenuToolRef.value) {
			onLuckySheetCreated(luckySheetMenuToolRef.value as HTMLElement)
		}
	}

	/**
	 * The function `onLuckySheetCreated` takes an HTML element as a parameter and sets up an observer to
	 * track its visibility.
	 * @param {HTMLElement} element - The element parameter is an HTMLElement object that represents the
	 * element on which the IntersectionObserver will be applied.
	 */
	const onLuckySheetCreated = (element: HTMLElement) => {
		menuToolVisibleObserver.value = new IntersectionObserver(handleIntersection)
		menuToolVisibleObserver.value.observe(element)
	}

	/**
	 * The `onMouseEnter` function sets the position of a tooltip based on the position of the target
	 * element.
	 * @param {MouseEvent} event - The event parameter is of type MouseEvent, which represents a mouse
	 * event. It contains information about the event, such as the target element that triggered the
	 * event.
	 */
	const onMouseEnter = (event: MouseEvent) => {
		const targetElement = event.target
		if (
			targetElement instanceof HTMLElement &&
			targetElement.matches('.luckysheet-toolbar-button, .luckysheet-toolbar-menu-button, .luckysheet-toolbar-combo-button')
		) {
			const rect = targetElement.getBoundingClientRect()
			toolTipPosition.value = {
				top: rect.top,
				left: rect.left,
				right: rect.right,
				bottom: rect.bottom,
				width: rect.width,
				x: rect.left + rect.width / 2,
				y: rect.top + rect.height
			}
		}
	}

	/**
	 * The function `handleIntersection` checks if an element is intersecting with the viewport and
	 * performs certain actions based on the result.
	 * @param {IntersectionObserverEntry[]} entries - The `entries` parameter is an array of
	 * `IntersectionObserverEntry` objects. Each `IntersectionObserverEntry` object represents the
	 * intersection between the observed target element and a specific ancestor element in the DOM.
	 */
	const handleIntersection = (entries: IntersectionObserverEntry[]) => {
		entries.forEach(entry => {
			if (entry.isIntersecting && luckySheetMenuToolRef.value) {
				setBodyStyle()
				luckySheetMenuToolRef.value.addEventListener('mouseenter', onMouseEnter, true)
				return
			}
			if (!isShow.value) {
				bodyClassList.remove(bodyClassName)
				luckySheetMenuToolRef.value?.removeEventListener('mouseenter', onMouseEnter, true)
			}
		})
	}

	/**
	 * The function `rewriteMenuTooltip` is used to observe changes in the child nodes of a parent element
	 * and call a handler function when a specific selector is found in the newly added child nodes.
	 * @param {string} parentSelector - The parentSelector parameter is a string that represents a CSS
	 * selector for the parent element. It is used to select the parent element in the DOM to which the
	 * MutationObserver will be attached.
	 * @param {string} unbornSelector - The `unbornSelector` parameter is a string that represents a CSS
	 * selector for the element(s) that will be created dynamically within the `parentElement`. This
	 * selector is used to identify the newly created elements and perform some actions on them.
	 */
	const rewriteMenuTooltip = (parentSelector: string, unbornSelector: string) => {
		// 获取父元素
		const parentElement = document.querySelector(parentSelector)

		// 创建MutationObserver实例，监听子节点变化
		luckySheetCreatObserver.value = new MutationObserver(mutationsList => handleLuckySheetCreat(mutationsList, unbornSelector))
		parentElement && luckySheetCreatObserver.value.observe(parentElement, { childList: true })
	}

	/**
	 * The function sets the position of a tooltip or menu element based on a given target element and a
	 * toolTipPosition value.
	 * @param {HTMLElement} targetElement - The targetElement parameter is an HTMLElement that represents
	 * the element to which the tooltip or menu should be positioned relative to.
	 * @returns The function does not explicitly return anything.
	 */
	const setTooltipAndMenuPosition = (targetElement: HTMLElement) => {
		if (toolTipPosition.value == null) {
			return
		}
		const tooltipOrMenuRect = targetElement.getBoundingClientRect()
		targetElement.style.left = toolTipPosition.value.x + 'px'
		targetElement.style.top = toolTipPosition.value.y + 'px'
		if (toolTipPosition.value.x > tooltipOrMenuRect.width / 2) {
			targetElement.style.transform = `translateX(-50%)`
		}
		const arrow = targetElement.children.item(1) as HTMLElement | null
		if (arrow?.matches('.jfk-tooltip-arrow, .jfk-tooltip-arrowup')) {
			arrow.style.left = '50%'
			arrow.style.transform = `translateX(-50%)`
		}
		targetElement.style.display = 'block'
	}

	/**
	 * The function `handleDialogObserver` is used to handle the observation of dialog elements and
	 * perform certain actions when a dialog element is detected.
	 * @param {Node} node - The `node` parameter is a reference to a DOM element. It is used to check if
	 * the element is a modal dialog and perform certain actions on it.
	 */
	const handleDialogObserver = (node: Node) => {
		if (node instanceof HTMLElement && node.matches('.luckysheet-modal-dialog')) {
			const allDialog = document.querySelectorAll('.luckysheet-modal-dialog-new')
			allDialog.forEach(dialog => {
				!dialog.hasChildNodes() && dialog.remove()
			})
			Array.from(node.children).forEach(child => {
				if (child && child instanceof HTMLElement) {
					child.style.pointerEvents = 'all'
				}
			})

			const customDialogWrap = document.createElement('div')
			customDialogWrap.setAttribute('draggable', 'true')
			customDialogWrap.classList.add('luckysheet-modal-dialog-new')
			customDialogWrap.style.display = 'block'
			customDialogWrap.appendChild(node)
			document.body.appendChild(customDialogWrap)

			useDraggable(customDialogWrap)
		}
	}

	// 点击或右击时，点击的或出现的菜单位置
	const menuRect = ref<DOMRect | null>(null)
	const luckySheetObserver = ref<MutationObserver | null>(null)
	const activeSheetRect = ref<DOMRect | null>(null)
	/**
	 * The `luckySheetObserver` function is a TypeScript function that sets up a mutation observer to
	 * listen for changes in a parent element and perform specific actions based on the mutations.
	 * @param {TEl} parentElement - The parent element to observe for mutations.
	 * @param {MutationObserverInit} config - The `config` parameter is an optional object that specifies
	 * the configuration options for the `MutationObserver`. It has the following properties:
	 */
	const handleLuckySheetObserver = <TEl extends Node = Node>(
		parentElement: TEl,
		config: MutationObserverInit = {
			attributes: true,
			attributeFilter: ['style'],
			childList: true,
			subtree: true
		}
	) => {
		luckySheetObserver.value = new MutationObserver(mutationsList => {
			mutationsList.forEach(mutation => {
				const targetElement = mutation.target
				if (mutation.type === 'childList' && mutation.addedNodes.length) {
					// 新增子节点
					mutation.addedNodes.forEach(node => {
						handleDialogObserver(node)
					})
					return
				}

				if (targetElement instanceof HTMLElement && mutation.type === 'attributes') {
					if (mutation.attributeName !== 'style') {
						return
					}
					// 样式属性变化
					const targetElStyle = targetElement.style
					if (targetElement.matches('#luckysheet-tooltip-up') && (targetElStyle.left === '2px' || targetElStyle.top === '29px')) {
						// 顶部工具栏的tooltip出现，且为初始样式
						setTooltipAndMenuPosition(targetElement)
						return
					}

					// excel右键菜单
					if (targetElement.matches('#luckysheet-rightclick-menu')) {
						menuRect.value = targetElement.getBoundingClientRect()
						return
					}

					// 顶部工具菜单
					const regex = /^luckysheet-icon(?:-[a-zA-Z]+)+-menuButton$/
					if (regex.test(targetElement.id) && targetElStyle.display === 'block') {
						// 顶部工具栏的子菜单出现 样式属性变化且 变化的属性为display，值为block
						if (Number(targetElStyle.left.replace('px', '')) < 1) {
							setTooltipAndMenuPosition(targetElement)
							menuRect.value = targetElement.getBoundingClientRect()
							return
						}
						if (targetElement.matches('.luckysheet-menuButton-sub') && Number(targetElStyle.top.replace('px', '')) < 1) {
							// 顶部条件格式子菜单
							targetElement.style.left = `${menuRect.value?.right ?? 200}px`
							targetElement.style.top = `${menuRect.value?.y ?? 0}px`
							targetElement.style.height = `${menuRect.value?.height ?? 500}px`
						}

						return
					}

					// excel右键菜单复制选项
					if (targetElement.matches('#luckysheetcopyfor_sub') && targetElStyle.display === 'block') {
						targetElement.style.left = `${menuRect.value?.right ?? 200}px`
						targetElement.style.top = `${menuRect.value?.y ?? 200}px`
						return
					}

					if (targetElement.matches('.luckysheet-modal-dialog') && targetElStyle.display === 'none') {
						if (targetElement.parentElement) {
							targetElement.parentElement.style.display = 'none'
						}
						return
					}

					if (targetElement.matches('#luckysheet-rightclick-sheet-menu') && targetElStyle.display === 'block') {
						// 底部sheet菜单
						const activeSheet = document.querySelector('.luckysheet-sheets-item.luckysheet-sheets-item-active')
						if (activeSheet != null && activeSheet instanceof HTMLElement) {
							activeSheetRect.value = activeSheet.getBoundingClientRect()
							targetElStyle.left = `${activeSheetRect.value.right - 20}px`
							menuRect.value = targetElement.getBoundingClientRect()
						}
						return
					}

					if (targetElement.matches('#luckysheetsheetconfigcolor_sub') && targetElStyle.display === 'block') {
						// 底部sheet菜单颜色选项
						targetElement.style.left = `${menuRect.value?.right}px`
						targetElement.style.top = `${(menuRect.value?.y ?? 300) + 50}px`
						return
					}
				}
			})
		})

		luckySheetObserver.value.observe(parentElement, config)
	}

	/**
	 * The function `getMenuToolPosition` is used to handle the positioning of a menu tooltip and observe
	 * changes in the LuckySheet editor.
	 */
	const getMenuToolPosition = () => {
		rewriteMenuTooltip('#myTemplatesLuckySheet', '#luckysheet-wa-editor')
		handleLuckySheetObserver(document.body)
	}

	onBeforeUnmount(() => {
		luckySheetObserver.value?.disconnect()
		luckySheetCreatObserver.value?.disconnect()
		menuToolVisibleObserver.value?.disconnect()
	})

	return {
		getMenuToolPosition
	}
}
